#include "../../include/database.h"
#include "../../include/error.h"
#include "../../include/util.h"
#include "../../include/ctx.h"

#include <string.h>

bool lm_ctx_database_init(lm_ctx_t *ctx){
  if(ctx->db != NULL)
    return true;

  if(NULL == ctx->data){
    lm_error_set(LM_ERR_CtxDataNULL);
    return false;
  }

  char dbpath[strlen(ctx->data)+10];
  join(dbpath, ctx->data, "db");

  ctx->db = lm_database_new(dbpath);
  return ctx->db != NULL; // error set by function
}

bool lm_ctx_database_is_installed(lm_ctx_t *ctx, char *name, char *version){
  if(!lm_ctx_database_init(ctx))
    return false; // error set by function

  if(NULL == ctx->db)
    return false;

  return lm_database_entry_find(ctx->db, NULL, name, version);
}

bool lm_ctx_database_find(lm_ctx_t *ctx, lm_entry_t *entry, char *name, char *version){
  if(NULL == ctx || NULL == name){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  if(!lm_ctx_database_init(ctx))
    return false; // error set by function

  lm_error_clear();
  return lm_database_entry_find(ctx->db, entry, name, version);
}

bool lm_ctx_database_next(lm_ctx_t *ctx, lm_entry_t *entry){
  if(NULL == ctx || NULL == entry){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  if(!lm_ctx_database_init(ctx))
    return false; // error set by function

  return lm_database_entry_next(ctx->db, entry);
}

bool lm_ctx_database_next_free(lm_ctx_t *ctx, lm_entry_t *entry){
  if(NULL == ctx || NULL == entry){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  if(!lm_ctx_database_init(ctx))
    return false; // error set by function

  lm_database_entry_next_free(ctx->db, entry);
  return true;
}

char *lm_ctx_database_changes(lm_ctx_t *ctx, lm_entry_t *entry){
  if(NULL == ctx || NULL == entry){
    lm_error_set(LM_ERR_ArgNULL);
    return NULL;
  }

  if(!lm_ctx_database_init(ctx))
    return NULL; // error set by function

  return lm_database_changes_get(ctx->db, entry);
}
