#include "../../include/package.h"
#include "../../include/error.h"
#include "../../include/util.h"
#include "../../include/ctx.h"

#include <stdbool.h>
#include <stdio.h>
#include <string.h>

bool lm_ctx_package_from(lm_ctx_t *ctx, lm_pkg_t *pkg, char *archive){
  if(NULL == ctx || NULL == pkg || NULL == archive){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  // we should init the package pointer as soon as possible
  // so if the caller calls ..._from_free we can prevent
  // memory corruption
  lm_package_init(pkg);

  if(NULL == ctx->temp) {
    lm_error_set(LM_ERR_CtxTempNULL);
    return false;
  }

  size_t archive_len = strlen(archive);
  char signature[archive_len+7];
  lm_pkg_files_t *files = NULL;
  bool ret = false;

  sprintf(signature, "%s.sig", archive);

  if(!lm_package_path_set_archive(pkg, archive) || !lm_package_path_set_signature(pkg, signature)){
    lm_error_set(LM_ERR_ArchiveSetFail);
    return false;
  }

  if(!lm_ctx_temp_clear(ctx)){
    pdebug(__func__, "failed to clear the temp directory: %s", lm_strerror());
    return false; // error set by function
  }

  if((files = lm_package_extract(pkg, ctx->temp)) == NULL){
    pdebug(__func__, "failed to extract the archive (%s): %s", archive, lm_strerror());
    return false; // error set by function
  }

  if(!lm_package_data_load(&pkg->data, files->data_file)){
    pdebug(__func__, "failed to load the package data: %s", lm_strerror());
    goto end; // error set by function
  }
  
  ret = true;
end:
  lm_package_extract_free(files);
  return ret;
}

void lm_ctx_package_from_free(lm_pkg_t *pkg){
  if(NULL == pkg){
    lm_error_set(LM_ERR_ArgNULL);
    return;
  }

  lm_package_free(pkg);
}
