#include "../../include/ctx.h"
#include "../../include/pool.h"
#include "../../include/util.h"
#include "../../include/error.h"

#include <stdbool.h>
#include <stdlib.h>
#include <string.h>

lm_pkg_t *lm_ctx_pool_find(lm_ctx_t *ctx, char *name, char *version){
  if(NULL == ctx || (NULL == name && NULL == version)){
    lm_error_set(LM_ERR_ArgNULL);
    return NULL;
  }

  lm_pool_t *cur = ctx->pools;
  lm_pkg_t *found = NULL;

  while(cur != NULL){
    if((found = lm_pool_package_find(cur, name, version)) != NULL)
      break;
    cur = cur->next;
  }

  if(NULL == found)
    lm_error_set(LM_ERR_PkgNotFound);

  return found;
}

lm_pool_t *lm_ctx_pool_add(lm_ctx_t *ctx, char *name, char *url, char *dir) {
  if(NULL == name || NULL == dir){
    lm_error_set(LM_ERR_ArgNULL);
    return NULL;
  }

  lm_pool_t *pool = lm_pool_new(name, url);
  if (NULL == pool)
    return NULL;

  if(NULL == url)
    pdebug(__func__, "adding new pool: %s", pool->name);
  else
    pdebug(__func__, "adding new pool: %s (%s://%s:%d%s)", pool->name, pool->url.protocol, pool->url.host, pool->url.port, pool->url.path);

  if(!lm_pool_path_set_dir(pool, dir)){
    // error is set by set_dir
    lm_pool_free(pool);
    return NULL;
  }

  // add to the start
  pool->next = ctx->pools;
  ctx->pools = pool;

  return pool;
}

void lm_ctx_pool_clear(lm_ctx_t *ctx) {
  lm_pool_t *cur = ctx->pools, *prev = NULL;
  while (NULL != cur) {
    prev = cur;
    cur  = cur->next;
    lm_pool_free(prev);
  }

  ctx->pools = NULL;
}

lm_pool_t *lm_ctx_pool_by_name(lm_ctx_t *ctx, char *name) {
  lm_pool_t *cur = ctx->pools;
  while (NULL != cur) {
    if (eq(cur->name, name))
      return cur;
    cur = cur->next;
  }
  return NULL;
}

lm_pool_t *lm_ctx_pool_by_url(lm_ctx_t *ctx, char *host, char *path) {
  lm_pool_t *cur = ctx->pools;
  while (NULL != cur) {
    if (eq(cur->url.host, host) && eq(cur->url.path, path))
      return cur;
    cur = cur->next;
  }
  return NULL;
}
