#include "../../include/package.h"
#include "../../include/error.h"
#include "../../include/util.h"

#include <stdlib.h>
#include <string.h>

ssize_t lm_package_data_depend_count(lm_pkg_data_t *data){
  size_t index = 0;

  if(NULL == data->depends)
    return index;

  while(data->depends[index] != NULL)
    index++;

  return index;
}

bool lm_package_data_depend_add(lm_pkg_data_t *data, char *depend){
  if(NULL == depend){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  if(!package_name_valid(depend)){
    lm_error_set(LM_ERR_PkgBadName);
    return false;
  }

  if(NULL == data->depends){
    data->depends = malloc(sizeof(char*)*2);
    data->depends[0] = strdup(depend);
    data->depends[1] = NULL;
    return true;
  }

  size_t count = lm_package_data_depend_count(data);
  data->depends = realloc(data->depends, sizeof(char*)*(count+2));
  data->depends[count++] = strdup(depend);
  data->depends[count] = NULL;
  return true;
}

void lm_package_data_depend_free(lm_pkg_data_t *data){
  if(NULL == data)
    return;

  if(NULL == data->depends)
    return;

  for(int i = 0; data->depends[i] != NULL; i++)
    free(data->depends[i]);
  free(data->depends);

  data->depends = NULL;
}

ssize_t lm_package_data_depend_strlen(lm_pkg_data_t *data){
  size_t len = 1;

  if(NULL == data->depends)
    return len;

  for(int i = 0; NULL != data->depends[i]; i++)
    len += strlen(data->depends[i])+1;

  return len;
}

char *lm_package_data_depend_tostr(lm_pkg_data_t *data, char *buffer){
  if(NULL == data){
    lm_error_set(LM_ERR_ArgNULL);
    return NULL;
  }

  if(NULL == buffer){
    size_t buflen = lm_package_data_depend_strlen(data)+1;
    buffer = malloc(buflen);
    bzero(buffer, buflen);
  }

  if(NULL == data->depends){
    buffer[0] = 0;
    return buffer;
  }

  size_t bufsz = 0, depsz = 0;

  for(int i = 0; NULL != data->depends[i]; i++){
    depsz = strlen(data->depends[i]);

    if(i == 0){
      memcpy(buffer, data->depends[i], depsz);
      bufsz += depsz;
      continue;
    }

    buffer[bufsz++] = ',';
    memcpy(buffer+bufsz, data->depends[i], depsz);
    bufsz += depsz;
  }

  buffer[bufsz] = 0;
  return buffer;
}

bool lm_package_data_depend_fromstr(lm_pkg_data_t *data, char *buffer){
  if(NULL == data){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  lm_package_data_depend_free(data);

  if(NULL == buffer)
    return true;

  char *save = NULL, *dep = NULL;
  dep = strtok_r(buffer, ",", &save);

  if(NULL == dep)
    return true;

  do {
    if(!lm_package_data_depend_add(data, dep))
      return false;
  }while((dep = strtok_r(NULL, ",", &save)) != NULL);

  return true;
}

bool lm_package_data_depends(lm_pkg_data_t *data, char *dep){
  if(NULL == data || NULL == dep){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  if(NULL == data->depends)
    return false;

  for(int i = 0; data->depends[i] != NULL; i++)
    if(eq(data->depends[i], dep))
      return true;
  return false;
}
