#include "../../include/package.h"
#include "../../include/error.h"
#include "../../include/mptp.h"
#include "../../include/util.h"

#include <errno.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <unistd.h>

lm_pkg_t *lm_package_new(){
  lm_pkg_t *pkg = malloc(sizeof(lm_pkg_t));
  lm_package_init(pkg);
  return pkg;
}

void lm_package_init(lm_pkg_t *pkg){
  bzero(pkg, sizeof(lm_pkg_t));
}

bool lm_package_downloaded(lm_pkg_t *pkg){
  if(lm_package_path_is_empty(pkg)){
    lm_error_set(LM_ERR_PkgPathsEmpty);
    return false;
  }

  if(!exists(pkg->archive, NULL) || !exists(pkg->signature, NULL)){
    lm_error_set(LM_ERR_PkgNotDownloaded);
    return false;
  }

  return true;
}

bool lm_package_remove_download(lm_pkg_t *pkg){
  if(unlink(pkg->archive) < 0 && errno != ENOENT){
    lm_error_set(LM_ERR_PkgRemoveDownloadFail);
    return false;
  }

  if(unlink(pkg->signature) < 0 && errno != ENOENT){
    lm_error_set(LM_ERR_PkgRemoveDownloadFail);
    return false;
  }

  return true;
}

bool lm_package_is_same(lm_pkg_t *one, lm_pkg_t *two){
  if(NULL == one || NULL == two){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  return eq(one->data.version, two->data.version) &&
         eq(one->data.name, two->data.name) &&
         eq(one->data.desc, two->data.desc) &&
         one->data.size == two->data.size;
}

void lm_package_free(lm_pkg_t *pkg){
  free(pkg->archive);
  free(pkg->signature);
  lm_package_data_free(&pkg->data);
  bzero(pkg, sizeof(lm_pkg_t));
}
