#include "../../include/package.h"
#include "../../include/error.h"
#include "../../include/util.h"

#include <stdlib.h>
#include <gpgme.h>

bool lm_package_verify(lm_pkg_t *pkg){
  if(NULL == pkg){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  // error set by function
  if(!lm_package_downloaded(pkg))
    return false;

  gpgme_data_t          sig = NULL, text = NULL;
  gpgme_ctx_t           ctx = NULL;
  gpgme_signature_t     s;
  gpgme_verify_result_t result;
  gpgme_error_t         err;
  bool ret = false;

  gpgme_check_version(NULL);
  err = gpgme_new(&ctx);

  if (err != GPG_ERR_NO_ERROR) {
    pdebug(__func__, "(%s) failed to create gpgme: %s", pkg->data.name, gpgme_strerror(err));
    lm_error_set(LM_ERR_PkgGPGFail);
    return false;
  }

  gpgme_op_import(ctx, NULL);

  if((err = gpgme_data_new_from_file(&sig, pkg->signature, 1)) != GPG_ERR_NO_ERROR){
    pdebug(__func__, "(%s) failed to import package signature %s", pkg->data.name, gpgme_strerror(err));
    lm_error_set(LM_ERR_PkgGPGSigFail);
    goto end;
  }

  if ((err = gpgme_data_new_from_file(&text, pkg->archive, 1)) != GPG_ERR_NO_ERROR) {
    pdebug(__func__, "(%s) failed to import package archive %s", pkg->data.name, gpgme_strerror(err));
    lm_error_set(LM_ERR_PkgGPGArchiveFail);
    goto end;
  }

  if ((err = gpgme_op_verify(ctx, sig, text, NULL)) != GPG_ERR_NO_ERROR) {
    pdebug(__func__, "(%s) failed to verify package archive %s", pkg->data.name, gpgme_strerror(err));
    lm_error_set(LM_ERR_PkgSigNoResult);
    goto end;
  }

  result = gpgme_op_verify_result(ctx);
  lm_error_set(LM_ERR_PkgSigNoMatch);

  if (result == NULL)
    goto end;

  s = result->signatures;
  while (s) {
    int status = gpg_err_code(s->status);
    if (status == GPG_ERR_NO_ERROR) {
      lm_error_set(LM_ERR_NoError);
      ret = true;
      break;
    }
    s = s->next;
  }

end:
  if(NULL != text)
    gpgme_data_release(text);
  
  if(NULL != sig)
    gpgme_data_release(sig);
  
  if(NULL != ctx)
    gpgme_release(ctx);

  return ret;
}
