#include "../../include/package.h"
#include "../../include/error.h"
#include "../../include/util.h"
#include "../../include/pool.h"

#include <stdbool.h>
#include <string.h>

lm_pkg_t *lm_pool_package_find(lm_pool_t *pool, char *name, char *version){
  if(NULL == name && NULL == version){
    lm_error_set(LM_ERR_ArgNULL);
    return NULL;
  }

  lm_pkg_t *cur = pool->pkg;
  while(NULL != cur){
    if(NULL == version && eq(cur->data.name, name))
      return cur;
    
    else if(NULL == name && eq(cur->data.version, version))
      return cur;
    
    else if(eq(cur->data.name, name) && eq(cur->data.version, version))
      return cur;

    cur = cur->next;
  }

  return NULL;
}

bool lm_pool_package_add(lm_pool_t *pool, lm_pkg_t *pkg){
  if(NULL == pool || NULL == pkg){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  pkg->pool = pool;

  if(!lm_pool_path_is_empty(pool)){
    size_t path_size = strlen(pool->dir)+strlen(pkg->data.name)+strlen(pkg->data.version);
    size_t name_size = path_size - strlen(pool->dir);

    char archive_path[path_size+10], sig_path[path_size+20];
    char archive_name[name_size+10], sig_name[name_size+20];

    sprintf(archive_name, "%s_%s.mpf", pkg->data.name, pkg->data.version);
    sprintf(sig_name, "%s_%s.mpf.sig", pkg->data.name, pkg->data.version);

    join(archive_path, pool->dir, archive_name);
    join(sig_path, pool->dir, sig_name);

    lm_package_path_set_archive(pkg, archive_path);
    lm_package_path_set_signature(pkg, sig_path);
  }

  // add to the start
  pkg->next = pool->pkg;
  pool->pkg = pkg;
  return true;
}
